"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _KnapsackTwigRenderer_src;
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnapsackTwigRenderer = void 0;
const renderers_1 = require("@knapsack/app/renderers");
const types_1 = require("@knapsack/types");
const twig_renderer_1 = __importDefault(require("@basalt/twig-renderer"));
const path_1 = require("path");
const file_utils_1 = require("@knapsack/file-utils");
const utils_1 = require("./utils");
const pkg = JSON.parse((0, file_utils_1.readFileSync)((0, path_1.join)(__dirname, '../package.json')));
const protoTemplatePath = (0, path_1.join)(__dirname, '../prototype-template/prototype-template.twig');
(0, file_utils_1.assertFileExists)(protoTemplatePath);
const protoTwigNamespace = 'knapsack-renderer-twig-prototype-template';
class KnapsackTwigRenderer extends renderers_1.RendererBase {
    constructor(config) {
        super({
            id: types_1.rendererIds.twig,
            language: 'twig',
            codeSrcsUserConfig: [],
        });
        _KnapsackTwigRenderer_src.set(this, void 0);
        this.render = async (opt) => {
            const usage = await this.getUsage(opt);
            const results = await this.twigRenderer.renderString(usage);
            return { ...results, usage, templateLanguage: this.language };
            // return this.twigRenderer.render(template.alias, data);
        };
        this.getUsage = async ({ demo, state }) => {
            if (!demo) {
                throw new Error(`No demo received`);
            }
            switch (demo.type) {
                case 'template': {
                    const { absolutePath: templateDemoPath, exists } = await this.resolvePath(demo.templateInfo.path);
                    if (!exists) {
                        throw new Error(`Could not find template demo file at ${templateDemoPath}`);
                    }
                    return (0, file_utils_1.readFile)(templateDemoPath);
                }
                case 'data-w-template-info':
                case 'data': {
                    const { namespacePath } = this.normalizeDemo({ demo, state });
                    const { data: { props, slots, slotsOptionsComputed }, } = demo;
                    if (!slots) {
                        return (0, utils_1.getTwigUsage)({
                            templateName: namespacePath,
                            props,
                        });
                    }
                    {
                        const slotProps = {};
                        await Promise.all(Object.keys(slots).map(async (slotName) => {
                            const slottedTemplates = slots[slotName];
                            slotProps[slotName] = await Promise.all(slottedTemplates.map(async (slottedTemplate) => {
                                if (slottedTemplate.type === 'text') {
                                    return `  ${slottedTemplate.text}`;
                                }
                                if (slottedTemplate.type === 'template-reference') {
                                    throw new Error(`Cannot handle slotted template references in ${JSON.stringify({
                                        demoId: demo.id,
                                        slottedTemplate,
                                    })}`);
                                }
                                const usage = await this.getUsage({
                                    demo: slottedTemplate.demo,
                                    state,
                                });
                                // indenting 2 spaces since it will be in a `{% set slotName %}` block
                                return usage
                                    .split('\n')
                                    .map((line) => `  ${line}`)
                                    .join('\n');
                            }));
                        }));
                        const extraProps = [];
                        const slotIncludes = Object.keys(slotProps).map((slotName) => {
                            extraProps.push({
                                key: slotName,
                                value: slotName,
                            });
                            const slotCodes = slotProps[slotName];
                            const slotOptionsComputed = slotsOptionsComputed === null || slotsOptionsComputed === void 0 ? void 0 : slotsOptionsComputed[slotName];
                            const { openTag, closeTag } = (0, renderers_1.createSlotOptionsHtmlTags)({
                                slotOptionsComputed,
                                classAttributeName: 'class',
                                stylesValueType: 'string',
                            });
                            return `
{% set ${slotName} %}
${[openTag, ...slotCodes, closeTag].filter(Boolean).join('\n')}
{% endset %}
      `.trim();
                        });
                        const usage = await (0, utils_1.getTwigUsage)({
                            templateName: namespacePath,
                            props,
                            before: slotIncludes.join('\n\n'),
                            extraProps,
                        });
                        return usage;
                    }
                }
                default: {
                    const _exhaustiveCheck = demo;
                    throw new Error(`Unknown demo type: ${JSON.stringify(demo)}`);
                }
            }
        };
        this.getDiscovery = async () => {
            var _a, _b, _c, _d;
            const [rootDir] = (_b = (_a = __classPrivateFieldGet(this, _KnapsackTwigRenderer_src, "f")) === null || _a === void 0 ? void 0 : _a.roots) !== null && _b !== void 0 ? _b : [];
            if (!rootDir) {
                return {
                    rendererId: this.id,
                    meta: this.getMeta(),
                    codeSrcsByPath: Object.fromEntries(this.codeSrcs.entries()),
                    templates: [],
                };
            }
            // `@namespace/path/to/file.twig`
            const twigPaths = await Promise.all((_c = __classPrivateFieldGet(this, _KnapsackTwigRenderer_src, "f").namespaces) === null || _c === void 0 ? void 0 : _c.map(async ({ id, paths }) => {
                const twigFiles = await (0, file_utils_1.globby)(paths
                    .map((path) => (0, path_1.isAbsolute)(path) ? path : (0, path_1.join)(this.userConfigDir, path))
                    .map((p) => (0, path_1.join)(p, '*.twig')), {
                    cwd: this.userConfigDir,
                    absolute: true,
                });
                return twigFiles.map((path) => {
                    const relativePath = (0, path_1.relative)(rootDir, path);
                    return (0, path_1.join)(`@${id}`, relativePath);
                });
            })).then((paths) => paths.flat());
            return {
                rendererId: this.id,
                meta: this.getMeta(),
                codeSrcsByPath: Object.fromEntries(this.codeSrcs.entries()),
                templates: (_d = twigPaths === null || twigPaths === void 0 ? void 0 : twigPaths.map((twigPath) => (0, types_1.addIdToTemplateInfo)({
                    namespacePath: twigPath,
                    rendererId: this.id,
                }))) !== null && _d !== void 0 ? _d : [],
            };
        };
        this.getTemplateSuggestions = async ({ state, }) => {
            var _a, _b;
            const { unusedPaths, usedPaths } = await this.getUnusedTemplatePaths({
                globPaths: __classPrivateFieldGet(this, _KnapsackTwigRenderer_src, "f").roots.map((root) => (0, path_1.join)(root, '**/*.twig')),
                cwd: this.userConfigDir,
                state,
            });
            const namespaces = (_b = (_a = __classPrivateFieldGet(this, _KnapsackTwigRenderer_src, "f").namespaces) === null || _a === void 0 ? void 0 : _a.map(({ id, paths }) => {
                const absPaths = paths.map((path) => (0, path_1.isAbsolute)(path) ? path : (0, path_1.join)(this.userConfigDir, path));
                return {
                    id,
                    paths: absPaths,
                };
            })) !== null && _b !== void 0 ? _b : [];
            const createSuggestion = (twigPath, connected) => {
                const namespace = namespaces.find(({ paths }) => paths.some((path) => twigPath.startsWith(path)));
                let alias;
                if (namespace) {
                    namespace.paths.forEach((path) => {
                        if (twigPath.startsWith(path)) {
                            const subPath = (0, path_1.relative)(path, twigPath);
                            alias = `@${namespace.id}/${subPath}`;
                        }
                    });
                }
                return {
                    path: twigPath,
                    alias,
                    connected,
                };
            };
            const unusedSuggestions = unusedPaths.map((path) => createSuggestion(path, false));
            const usedSuggestions = usedPaths.map((path) => createSuggestion(path, true));
            return { suggestions: [...unusedSuggestions, ...usedSuggestions] };
        };
        this.getMeta = () => ({
            id: this.id,
            title: 'Twig',
            aliasUse: 'optional',
            aliasTitle: 'Namespaced Path',
            aliasDescription: "If using Twig Namespaces (i.e. `@components`) then provide the value you'd provide to a Twig `include`, such as `@components/card.twig`.",
            enableDataDemos: true,
            enableTemplateDemos: true,
            hasSlotsSupport: true,
            hasSlotOptionsSupport: true,
            version: pkg.version,
            hasInferSpecSupport: false,
            hasTemplateSuggestionsSupport: true,
            prototypingTemplate: {
                path: '',
                alias: `@${protoTwigNamespace}/${(0, path_1.basename)(protoTemplatePath)}`,
                spec: {
                    isInferred: false,
                    props: {
                        type: 'object',
                        properties: {},
                    },
                    slots: {
                        children: { title: 'Children' },
                    },
                },
            },
        });
        // copying b/c `TwigRenderer` mutates config!
        __classPrivateFieldSet(this, _KnapsackTwigRenderer_src, JSON.parse(JSON.stringify(config.src)), "f");
        this.twigRenderer = new twig_renderer_1.default({
            keepAlive: process.env.NODE_ENV === 'production',
            ...config,
            src: {
                ...config.src,
                namespaces: [
                    ...config.src.namespaces,
                    {
                        id: protoTwigNamespace,
                        paths: [(0, path_1.dirname)(protoTemplatePath)],
                    },
                ],
            },
        });
    }
}
exports.KnapsackTwigRenderer = KnapsackTwigRenderer;
_KnapsackTwigRenderer_src = new WeakMap();
//# sourceMappingURL=renderer-twig.js.map